==> flutter_mylibrary-0.2.3.tar.gz: lib/base/provider/view_state_model.dart <==
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_mylibrary/generated/i18n.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:flutter_mylibrary/widget/oktoast/core/toast.dart';
import '../../flutter_mylibrary.dart';
import 'view_state.dart';
export 'view_state.dart';


typedef ViewStateModelWidgetBuilder<T> = Widget Function(T value);


abstract class ViewStateModel with ChangeNotifier {


  /// 防止页面销毁后,异步任务才完成,导致报错
  bool _disposed = false;

  /// 当前的页面状态,默认为busy,可在viewModel的构造方法中指定;
  ViewState _viewState;

  EasyRefreshController _refreshController = EasyRefreshController();

  EasyRefreshController get refreshController => _refreshController;


  bool isInit = false;

  /// 根据状态构造
  ///
  /// 子类可以在构造函数指定需要的页面状态
  /// FooModel():super(viewState:ViewState.busy);
  ViewStateModel({ViewState viewState})
      : _viewState = viewState ?? ViewState.busy {
    debugPrint('ViewStateModel---constructor--->$runtimeType');
  }


  /// ViewState
  ViewState get viewState => _viewState;

  set viewState(ViewState viewState) {
    _viewStateError = null;
    _viewState = viewState;
    notifyListeners();
  }

  /// ViewStateError
  ViewStateError _viewStateError;

  ViewStateError get viewStateError => _viewStateError;

  /// 以下变量是为了代码书写方便,加入的get方法.严格意义上讲,并不严谨
  ///
  /// get
  bool get isBusy => viewState == ViewState.busy;

  bool get isIdle => viewState == ViewState.idle;

  bool get isEmpty => viewState == ViewState.empty;

  bool get isError => viewState == ViewState.error;

  /// set
  void setIdle() {
    viewState = ViewState.idle;
    isInit = true;
  }

  void setBusy() {
    viewState = ViewState.busy;
  }

  void setEmpty() {
    viewState = ViewState.empty;
  }

  /// [e]分类Error和Exception两种
  void setError(e, stackTrace, {String message}) {
    ViewStateErrorType errorType = ViewStateErrorType.defaultError;

    /// 见https://github.com/flutterchina/dio/blob/master/README-ZH.md#dioerrortype
    if (e is HttpError) {
      message = e.message;
    }
    viewState = ViewState.error;
    _viewStateError = ViewStateError(
      errorType,
      message: message,
      errorMessage: e.toString(),
    );
    printErrorStack(e, stackTrace);
    onError(viewStateError);
  }

  void onError(ViewStateError viewStateError) {}



  /// 显示错误消息
  showErrorMessage(context, {String message}) {
    if (viewStateError != null || message != null) {
      if (viewStateError.isNetworkTimeOut) {
        message ??= S.of(context).viewStateMessageNetworkError;
      } else {
        message ??= viewStateError.message;
      }
      Future.microtask(() {
        showToast(message, context: context);
      });
    }
  }

  @override
  String toString() {
    return 'BaseModel{_viewState: $viewState, _viewStateError: $_viewStateError}';
  }

  @override
  void notifyListeners() {
    if (!_disposed) {
      super.notifyListeners();
    }
  }

  @override
  void dispose() {
    _disposed = true;
    debugPrint('view_state_model dispose -->$runtimeType');
    if (isNotEmpty(_instance)) {
      _ViewModelStore._of().disRegister(instanceName: _instance);
    }else{
      _ViewModelStore._of().disRegister(type: runtimeType);
    }
    _instance = '';
    super.dispose();
  }

  static String _instance;



  ///传入对应的ViewModel泛型
  ///[builder] 实例化view model
  ///
  /// 1. 取当前嵌套页面组件VM type数据 [ancestorOfType]=false
  /// 2. 取根页面组件VM type数据 [ancestorOfType]=true&&[builder]==null
  /// 3. 取当前页面存在VM type数据 [ancestorOfType]=false
  static VM of<VM extends ViewStateModel>({VM Function() builder, String instance, bool ancestorOfType = false}) {
    _instance = instance;
    VM vm = _ViewModelStore._of().register<VM>(builder: builder, instanceName: instance);
    assert(vm != null, 'do you forget register the viewModel ${vm.runtimeType} in embed pager or top pager?');
    return vm;
  }
}

/// [e]为错误类型 :可能为 Error , Exception ,String
/// [s]为堆栈信息
printErrorStack(e, s) {
  debugPrint('''
<-----↓↓↓↓↓↓↓↓↓↓-----error-----↓↓↓↓↓↓↓↓↓↓----->
$e
<-----↑↑↑↑↑↑↑↑↑↑-----error-----↑↑↑↑↑↑↑↑↑↑----->''');
//   if (s != null) debugPrint('''
// <-----↓↓↓↓↓↓↓↓↓↓-----trace-----↓↓↓↓↓↓↓↓↓↓----->
// $s
// <-----↑↑↑↑↑↑↑↑↑↑-----trace-----↑↑↑↑↑↑↑↑↑↑----->
//     ''');
}



///内置一个provider管理者
class _ViewModelStore{

  ///用于存储状态
  final _factories = Map<Type, dynamic>();

  /// the ones that get registered by name.
  final _factoriesByName = Map<String, dynamic>();

  _ViewModelStore();

  static _ViewModelStore _viewModelStore;

  static _ViewModelStore _of(){
    if (_viewModelStore == null) {
      _viewModelStore = _ViewModelStore();
    }
    return _viewModelStore;
  }


  void disRegister({Type type, String instanceName}){
    if (isNotEmpty(type)) {
      if (isRegister(viewModelType: type)) {
        _factories.remove(type);
      }
    } else if (isNotEmpty(instanceName)) {
      if (isRegister(instanceName: instanceName)) {
        _factoriesByName.remove(instanceName);
      }
    }
  }


  _clear() {
    _factories.values.forEach((value) {
      (value as ViewStateModel).dispose();
    });
    _factoriesByName.values.forEach((value) {
      (value as ViewStateModel).dispose();
    });
    _factories.clear();
    _factoriesByName.clear();
  }


  ///是否注册在provider管理器里
  bool isRegister<T extends ViewStateModel>({Type viewModelType, String instanceName}) {
    if (instanceName != null) {
      return _factoriesByName.containsKey(instanceName);
    } else if(viewModelType != null) {
      return _factories.containsKey(viewModelType);
    }
    return false;
  }


  ///泛型务必要加上
  ///如果不想使用该方法进行注册view model 请直接使用 [get]
  T register<T extends ViewStateModel>({String instanceName, @required T Function() builder}) {
    T viewModel;
    if (instanceName != null) {
      if (_factoriesByName.containsKey(instanceName)) {
        viewModel = _factoriesByName[instanceName];
        return viewModel;
      } else {
        viewModel = builder();
        debugPrint('register viewModel: $viewModel instanceName: $instanceName');
        _factoriesByName[instanceName] = viewModel;
        return viewModel;
      }
    } else {
      if (_factories.containsKey(T)) {
        viewModel = _factories[T];
        return viewModel;
      } else {
        viewModel = builder();
        debugPrint('register viewModel: $viewModel instanceName: $instanceName');
        _factories[T] = viewModel;
        return viewModel;
      }
    }
  }



}

==> flutter_mylibrary-0.2.4.tar.gz: lib/base/provider/view_state_model.dart <==
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_mylibrary/generated/i18n.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:flutter_mylibrary/widget/oktoast/core/toast.dart';
import '../../flutter_mylibrary.dart';
import 'view_state.dart';
export 'view_state.dart';


typedef ViewStateModelWidgetBuilder<T> = Widget Function(T value);


abstract class ViewStateModel with ChangeNotifier {


  /// 防止页面销毁后,异步任务才完成,导致报错
  bool _disposed = false;

  /// 当前的页面状态,默认为busy,可在viewModel的构造方法中指定;
  ViewState _viewState;

  EasyRefreshController _refreshController = EasyRefreshController();

  EasyRefreshController get refreshController => _refreshController;


  bool isInit = false;

  /// 根据状态构造
  ///
  /// 子类可以在构造函数指定需要的页面状态
  /// FooModel():super(viewState:ViewState.busy);
  ViewStateModel({ViewState viewState})
      : _viewState = viewState ?? ViewState.busy {
    debugPrint('ViewStateModel---constructor--->$runtimeType');
  }


  /// ViewState
  ViewState get viewState => _viewState;

  set viewState(ViewState viewState) {
    _viewStateError = null;
    _viewState = viewState;
    notifyListeners();
  }

  /// ViewStateError
  ViewStateError _viewStateError;

  ViewStateError get viewStateError => _viewStateError;

  /// 以下变量是为了代码书写方便,加入的get方法.严格意义上讲,并不严谨
  ///
  /// get
  bool get isBusy => viewState == ViewState.busy;

  bool get isIdle => viewState == ViewState.idle;

  bool get isEmpty => viewState == ViewState.empty;

  bool get isError => viewState == ViewState.error;

  /// set
  void setIdle() {
    viewState = ViewState.idle;
    isInit = true;
  }

  void setBusy() {
    viewState = ViewState.busy;
  }

  void setEmpty() {
    viewState = ViewState.empty;
  }

  /// [e]分类Error和Exception两种
  void setError(e, stackTrace, {String message}) {
    ViewStateErrorType errorType = ViewStateErrorType.defaultError;

    /// 见https://github.com/flutterchina/dio/blob/master/README-ZH.md#dioerrortype
    if (e is HttpError) {
      message = e.message;
    }
    viewState = ViewState.error;
    _viewStateError = ViewStateError(
      errorType,
      message: message,
      errorMessage: e.toString(),
    );
    printErrorStack(e, stackTrace);
    onError(viewStateError);
  }

  void onError(ViewStateError viewStateError) {}



  /// 显示错误消息
  showErrorMessage(context, {String message}) {
    if (viewStateError != null || message != null) {
      if (viewStateError.isNetworkTimeOut) {
        message ??= S.of(context).viewStateMessageNetworkError;
      } else {
        message ??= viewStateError.message;
      }
      Future.microtask(() {
        showToast(message, context: context);
      });
    }
  }

  @override
  String toString() {
    return 'BaseModel{_viewState: $viewState, _viewStateError: $_viewStateError}';
  }

  @override
  void notifyListeners() {
    if (!_disposed) {
      super.notifyListeners();
    }
  }

  @override
  void dispose() {
    _disposed = true;
    debugPrint('view_state_model dispose -->$runtimeType');
    _ViewModelStore._of().disRegister(this);
    super.dispose();
  }



  ///传入对应的ViewModel泛型
  ///[builder] 实例化view model
  ///
  /// 1. 取当前嵌套页面组件VM type数据 [ancestorOfType]=false
  /// 2. 取根页面组件VM type数据 [ancestorOfType]=true&&[builder]==null
  /// 3. 取当前页面存在VM type数据 [ancestorOfType]=false
  static VM of<VM extends ViewStateModel>({VM Function() builder, String instance, bool ancestorOfType = false}) {
    VM vm = _ViewModelStore._of().register<VM>(builder: builder, instanceName: instance);
    builder.hashCode;
    assert(vm != null, 'do you forget register the viewModel ${vm.runtimeType} in embed pager or top pager?');
    return vm;
  }

}

/// [e]为错误类型 :可能为 Error , Exception ,String
/// [s]为堆栈信息
printErrorStack(e, s) {
  debugPrint('''
<-----↓↓↓↓↓↓↓↓↓↓-----error-----↓↓↓↓↓↓↓↓↓↓----->
$e
<-----↑↑↑↑↑↑↑↑↑↑-----error-----↑↑↑↑↑↑↑↑↑↑----->''');
//   if (s != null) debugPrint('''
// <-----↓↓↓↓↓↓↓↓↓↓-----trace-----↓↓↓↓↓↓↓↓↓↓----->
// $s
// <-----↑↑↑↑↑↑↑↑↑↑-----trace-----↑↑↑↑↑↑↑↑↑↑----->
//     ''');
}



///内置一个provider管理者
class _ViewModelStore{

  ///用于存储状态
  final _factories = Map<Type, dynamic>();

  /// the ones that get registered by name.
  final _factoriesByName = Map<String, dynamic>();

  _ViewModelStore();

  static _ViewModelStore _viewModelStore;

  static _ViewModelStore _of(){
    if (_viewModelStore == null) {
      _viewModelStore = _ViewModelStore();
    }
    return _viewModelStore;
  }



  void disRegister(ViewStateModel viewStateModel){
    if (_factories.containsValue(viewStateModel)) {
      _factories.removeWhere((key, value) => value == viewStateModel);
    }else if (_factoriesByName.containsValue(viewStateModel)) {
      _factoriesByName.removeWhere((key, value) => value == viewStateModel);
    }
  }


  _clear() {
    _factories.values.forEach((value) {
      (value as ViewStateModel).dispose();
    });
    _factoriesByName.values.forEach((value) {
      (value as ViewStateModel).dispose();
    });
    _factories.clear();
    _factoriesByName.clear();
  }


  ///是否注册在provider管理器里
  bool isRegister<T extends ViewStateModel>({Type viewModelType, String instanceName}) {
    if (instanceName != null) {
      return _factoriesByName.containsKey(instanceName);
    } else if(viewModelType != null) {
      return _factories.containsKey(viewModelType);
    }
    return false;
  }


  ///泛型务必要加上
  ///如果不想使用该方法进行注册view model 请直接使用 [get]
  T register<T extends ViewStateModel>({String instanceName, @required T Function() builder}) {
    T viewModel;
    if (instanceName != null) {
      if (_factoriesByName.containsKey(instanceName)) {
        viewModel = _factoriesByName[instanceName];
        return viewModel;
      } else {
        viewModel = builder();
        debugPrint('register viewModel: $viewModel instanceName: $instanceName');
        _factoriesByName[instanceName] = viewModel;
        return viewModel;
      }
    } else {
      if (_factories.containsKey(T)) {
        viewModel = _factories[T];
        return viewModel;
      } else {
        viewModel = builder();
        debugPrint('register viewModel: $viewModel instanceName: $instanceName');
        _factories[T] = viewModel;
        return viewModel;
      }
    }
  }



}
